@@ -32,15 +32,16 @@ import _pytest
 from _pytest._io.saferepr import safeformat
 from _pytest._io.saferepr import saferepr
 from _pytest.compat import overload
+from _pytest.compat import TYPE_CHECKING
 
-if False:  # TYPE_CHECKING
+if TYPE_CHECKING:
     from typing import Type
     from typing_extensions import Literal
     from weakref import ReferenceType  # noqa: F401
 
     from _pytest._code import Source
 
-    _TracebackStyle = Literal["long", "short", "no", "native"]
+    _TracebackStyle = Literal["long", "short", "line", "no", "native"]
 
 
 class Code:
@@ -66,7 +67,7 @@ class Code:
         return not self == other
 
     @property
-    def path(self):
+    def path(self) -> Union[py.path.local, str]:
         """ return a path object pointing to source code (note that it
         might not point to an actually existing file). """
         try:
@@ -334,7 +335,7 @@ class Traceback(List[TracebackEntry]):
                 (path is None or codepath == path)
                 and (
                     excludepath is None
-                    or not hasattr(codepath, "relto")
+                    or not isinstance(codepath, py.path.local)
                     or not codepath.relto(excludepath)
                 )
                 and (lineno is None or x.lineno == lineno)
@@ -479,7 +480,7 @@ class ExceptionInfo(Generic[_E]):
         assert tup[1] is not None, "no current exception"
         assert tup[2] is not None, "no current exception"
         exc_info = (tup[0], tup[1], tup[2])
-        return cls.from_exc_info(exc_info, exprinfo)
+        return ExceptionInfo.from_exc_info(exc_info, exprinfo)
 
     @classmethod
     def for_later(cls) -> "ExceptionInfo[_E]":
@@ -920,7 +921,7 @@ class TerminalRepr:
     def __repr__(self) -> str:
         return "<{} instance at {:0x}>".format(self.__class__, id(self))
 
-    def toterminal(self, tw) -> None:
+    def toterminal(self, tw: py.io.TerminalWriter) -> None:
         raise NotImplementedError()
 
 
@@ -931,7 +932,7 @@ class ExceptionRepr(TerminalRepr):
     def addsection(self, name: str, content: str, sep: str = "-") -> None:
         self.sections.append((name, content, sep))
 
-    def toterminal(self, tw) -> None:
+    def toterminal(self, tw: py.io.TerminalWriter) -> None:
         for name, content, sep in self.sections:
             tw.sep(sep, name)
             tw.line(content)
@@ -951,7 +952,7 @@ class ExceptionChainRepr(ExceptionRepr):
         self.reprtraceback = chain[-1][0]
         self.reprcrash = chain[-1][1]
 
-    def toterminal(self, tw) -> None:
+    def toterminal(self, tw: py.io.TerminalWriter) -> None:
         for element in self.chain:
             element[0].toterminal(tw)
             if element[2] is not None:
@@ -968,7 +969,7 @@ class ReprExceptionInfo(ExceptionRepr):
         self.reprtraceback = reprtraceback
         self.reprcrash = reprcrash
 
-    def toterminal(self, tw) -> None:
+    def toterminal(self, tw: py.io.TerminalWriter) -> None:
         self.reprtraceback.toterminal(tw)
         super().toterminal(tw)
 
@@ -986,7 +987,7 @@ class ReprTraceback(TerminalRepr):
         self.extraline = extraline
         self.style = style
 
-    def toterminal(self, tw) -> None:
+    def toterminal(self, tw: py.io.TerminalWriter) -> None:
         # the entries might have different styles
         for i, entry in enumerate(self.reprentries):
             if entry.style == "long":
@@ -1018,7 +1019,7 @@ class ReprEntryNative(TerminalRepr):
     def __init__(self, tblines: Sequence[str]) -> None:
         self.lines = tblines
 
-    def toterminal(self, tw) -> None:
+    def toterminal(self, tw: py.io.TerminalWriter) -> None:
         tw.write("".join(self.lines))
 
 
@@ -1037,7 +1038,7 @@ class ReprEntry(TerminalRepr):
         self.reprfileloc = filelocrepr
         self.style = style
 
-    def toterminal(self, tw) -> None:
+    def toterminal(self, tw: py.io.TerminalWriter) -> None:
         if self.style == "short":
             assert self.reprfileloc is not None
             self.reprfileloc.toterminal(tw)
@@ -1070,7 +1071,7 @@ class ReprFileLocation(TerminalRepr):
         self.lineno = lineno
         self.message = message
 
-    def toterminal(self, tw) -> None:
+    def toterminal(self, tw: py.io.TerminalWriter) -> None:
         # filename and lineno output for each entry,
         # using an output format that most editors understand
         msg = self.message
@@ -1085,7 +1086,7 @@ class ReprLocals(TerminalRepr):
     def __init__(self, lines: Sequence[str]) -> None:
         self.lines = lines
 
-    def toterminal(self, tw) -> None:
+    def toterminal(self, tw: py.io.TerminalWriter) -> None:
         for line in self.lines:
             tw.line(line)
 
@@ -1094,7 +1095,7 @@ class ReprFuncArgs(TerminalRepr):
     def __init__(self, args: Sequence[Tuple[str, object]]) -> None:
         self.args = args
 
-    def toterminal(self, tw) -> None:
+    def toterminal(self, tw: py.io.TerminalWriter) -> None:
         if self.args:
             linesofar = ""
             for name, value in self.args:
