@@ -143,10 +143,12 @@ class AssertionRewritingHook(importlib.abc.MetaPathFinder):
         exec(co, module.__dict__)
 
     def _early_rewrite_bailout(self, name, state):
-        """This is a fast way to get out of rewriting modules. Profiling has
-        shown that the call to PathFinder.find_spec (inside of the find_spec
-        from this class) is a major slowdown, so, this method tries to
-        filter what we're sure won't be rewritten before getting to it.
+        """This is a fast way to get out of rewriting modules.
+
+        Profiling has shown that the call to PathFinder.find_spec (inside of
+        the find_spec from this class) is a major slowdown, so, this method
+        tries to filter what we're sure won't be rewritten before getting to
+        it.
         """
         if self.session is not None and not self._session_paths_checked:
             self._session_paths_checked = True
@@ -1043,14 +1045,13 @@ def try_makedirs(cache_dir) -> bool:
 
 def get_cache_dir(file_path: Path) -> Path:
     """Returns the cache directory to write .pyc files for the given .py file path"""
-    # Type ignored until added in next mypy release.
-    if sys.version_info >= (3, 8) and sys.pycache_prefix:  # type: ignore
+    if sys.version_info >= (3, 8) and sys.pycache_prefix:
         # given:
         #   prefix = '/tmp/pycs'
         #   path = '/home/user/proj/test_app.py'
         # we want:
         #   '/tmp/pycs/home/user/proj'
-        return Path(sys.pycache_prefix) / Path(*file_path.parts[1:-1])  # type: ignore
+        return Path(sys.pycache_prefix) / Path(*file_path.parts[1:-1])
     else:
         # classic pycache directory
         return file_path.parent / "__pycache__"
