@@ -27,7 +27,13 @@ from _pytest._io.saferepr import saferepr
 from _pytest.outcomes import fail
 from _pytest.outcomes import TEST_OUTCOME
 
-if False:  # TYPE_CHECKING
+if sys.version_info < (3, 5, 2):
+    TYPE_CHECKING = False  # type: bool
+else:
+    from typing import TYPE_CHECKING
+
+
+if TYPE_CHECKING:
     from typing import Type  # noqa: F401 (used in type string)
 
 
@@ -43,8 +49,7 @@ MODULE_NOT_FOUND_ERROR = (
 
 
 if sys.version_info >= (3, 8):
-    # Type ignored until next mypy release.
-    from importlib import metadata as importlib_metadata  # type: ignore
+    from importlib import metadata as importlib_metadata
 else:
     import importlib_metadata  # noqa: F401
 
@@ -92,8 +97,10 @@ def getlocation(function, curdir=None) -> str:
     function = get_real_func(function)
     fn = py.path.local(inspect.getfile(function))
     lineno = function.__code__.co_firstlineno
-    if curdir is not None and fn.relto(curdir):
-        fn = fn.relto(curdir)
+    if curdir is not None:
+        relfn = fn.relto(curdir)
+        if relfn:
+            return "%s:%d" % (relfn, lineno + 1)
     return "%s:%d" % (fn, lineno + 1)
 
 
@@ -372,7 +379,7 @@ class CaptureIO(io.TextIOWrapper):
         return self.buffer.getvalue().decode("UTF-8")
 
 
-if sys.version_info < (3, 5, 2):  # pragma: no cover
+if sys.version_info < (3, 5, 2):
 
     def overload(f):  # noqa: F811
         return f
@@ -385,9 +392,7 @@ else:
 
 
 if sys.version_info >= (3, 8):
-    # TODO: Remove type ignore on next mypy update.
-    # https://github.com/python/typeshed/commit/add0b5e930a1db16560fde45a3b710eefc625709
-    from functools import cached_property  # type: ignore
+    from functools import cached_property
 else:
 
     class cached_property(Generic[_S, _T]):
