@@ -5,9 +5,10 @@ from typing import Optional
 import py
 
 from .exceptions import UsageError
+from _pytest.compat import TYPE_CHECKING
 from _pytest.outcomes import fail
 
-if False:
+if TYPE_CHECKING:
     from . import Config  # noqa: F401
 
 
@@ -108,7 +109,7 @@ CFG_PYTEST_SECTION = "[pytest] section in {filename} files is no longer supporte
 
 
 def determine_setup(
-    inifile: str,
+    inifile: Optional[str],
     args: List[str],
     rootdir_cmd_arg: Optional[str] = None,
     config: Optional["Config"] = None,
@@ -120,7 +121,9 @@ def determine_setup(
         sections = ["tool:pytest", "pytest"] if is_cfg_file else ["pytest"]
         for section in sections:
             try:
-                inicfg = iniconfig[section]
+                inicfg = iniconfig[
+                    section
+                ]  # type: Optional[py.iniconfig._SectionWrapper]
                 if is_cfg_file and section == "pytest" and config is not None:
                     fail(
                         CFG_PYTEST_SECTION.format(filename=str(inifile)), pytrace=False
