@@ -13,18 +13,21 @@ from typing import Sequence
 from typing import Tuple
 from typing import Union
 
+import py
+
 import pytest
 from _pytest import outcomes
 from _pytest._code.code import ExceptionInfo
 from _pytest._code.code import ReprFileLocation
 from _pytest._code.code import TerminalRepr
 from _pytest.compat import safe_getattr
+from _pytest.compat import TYPE_CHECKING
 from _pytest.fixtures import FixtureRequest
 from _pytest.outcomes import Skipped
 from _pytest.python_api import approx
 from _pytest.warning_types import PytestWarning
 
-if False:  # TYPE_CHECKING
+if TYPE_CHECKING:
     import doctest
     from typing import Type
 
@@ -136,7 +139,7 @@ class ReprFailDoctest(TerminalRepr):
     ):
         self.reprlocation_lines = reprlocation_lines
 
-    def toterminal(self, tw) -> None:
+    def toterminal(self, tw: py.io.TerminalWriter) -> None:
         for reprlocation, lines in self.reprlocation_lines:
             for line in lines:
                 tw.line(line)
@@ -435,7 +438,19 @@ class DoctestModule(pytest.Module):
             https://bugs.python.org/issue25532
             """
 
-            def _find(self, tests, obj, name, module, source_lines, globs, seen):
+            def _find_lineno(self, obj, source_lines):
+                """
+                Doctest code does not take into account `@property`, this is a hackish way to fix it.
+
+                https://bugs.python.org/issue17446
+                """
+                if isinstance(obj, property):
+                    obj = getattr(obj, "fget", obj)
+                return doctest.DocTestFinder._find_lineno(self, obj, source_lines)
+
+            def _find(
+                self, tests, obj, name, module, source_lines, globs, seen
+            ) -> None:
                 if _is_mocked(obj):
                     return
                 with _patch_unwrap_mock_aware():
