@@ -28,6 +28,7 @@ from _pytest._code import Source
 from _pytest._io.saferepr import saferepr
 from _pytest.capture import MultiCapture
 from _pytest.capture import SysCapture
+from _pytest.compat import TYPE_CHECKING
 from _pytest.fixtures import FixtureRequest
 from _pytest.main import ExitCode
 from _pytest.main import Session
@@ -35,9 +36,11 @@ from _pytest.monkeypatch import MonkeyPatch
 from _pytest.pathlib import Path
 from _pytest.reports import TestReport
 
-if False:  # TYPE_CHECKING
+if TYPE_CHECKING:
     from typing import Type
 
+    import pexpect
+
 
 IGNORE_PAM = [  # filenames added when obtaining details about the current user
     "/var/lib/sss/mc/passwd"
@@ -189,7 +192,7 @@ class ParsedCall:
         del d["_name"]
         return "<ParsedCall {!r}(**{!r})>".format(self._name, d)
 
-    if False:  # TYPE_CHECKING
+    if TYPE_CHECKING:
         # The class has undetermined attributes, this tells mypy about it.
         def __getattr__(self, key):
             raise NotImplementedError()
@@ -433,9 +436,14 @@ class RunResult:
         for line in reversed(self.outlines):
             if rex_session_duration.search(line):
                 outcomes = rex_outcome.findall(line)
-                return {noun: int(count) for (count, noun) in outcomes}
-
-        raise ValueError("Pytest terminal summary report not found")
+                ret = {noun: int(count) for (count, noun) in outcomes}
+                break
+        else:
+            raise ValueError("Pytest terminal summary report not found")
+        if "errors" in ret:
+            assert "error" not in ret
+            ret["error"] = ret.pop("errors")
+        return ret
 
     def assert_outcomes(
         self,
@@ -827,7 +835,7 @@ class Testdir:
         items = [x.item for x in rec.getcalls("pytest_itemcollected")]
         return items, rec
 
-    def inline_run(self, *args, plugins=(), no_reraise_ctrlc=False):
+    def inline_run(self, *args, plugins=(), no_reraise_ctrlc: bool = False):
         """Run ``pytest.main()`` in-process, returning a HookRecorder.
 
         Runs the :py:func:`pytest.main` function to run all of pytest inside
@@ -1234,7 +1242,9 @@ class Testdir:
         args = self._getpytestargs() + args
         return self.run(*args, timeout=timeout)
 
-    def spawn_pytest(self, string, expect_timeout=10.0):
+    def spawn_pytest(
+        self, string: str, expect_timeout: float = 10.0
+    ) -> "pexpect.spawn":
         """Run pytest using pexpect.
 
         This makes sure to use the right pytest and sets up the temporary
@@ -1248,7 +1258,7 @@ class Testdir:
         cmd = "{} --basetemp={} {}".format(invoke, basetemp, string)
         return self.spawn(cmd, expect_timeout=expect_timeout)
 
-    def spawn(self, cmd, expect_timeout=10.0):
+    def spawn(self, cmd: str, expect_timeout: float = 10.0) -> "pexpect.spawn":
         """Run a command using pexpect.
 
         The pexpect child is returned.
